#! /bin/csh
############################################################################
# $CSURF_DIR/FreeSurferDefs.csh: setup environment for FreeSurfer (csh,tsch)
############################################################################

### need these 3 to run FreeSurfer programs (user .cshrc setenv's override)
if (! $?CSURF_DIR) then
  echo \
    "FreeSurferDefs.csh: ### CSURF_DIR not set--using /usr/local/FreeSurfer0.8"
  setenv CSURF_DIR /usr/local/FreeSurfer0.8
endif
if (! $?SUBJECTS_DIR)    setenv SUBJECTS_DIR     $CSURF_DIR/subjects
if (! $?FUNCTIONALS_DIR) setenv FUNCTIONALS_DIR  $CSURF_DIR/sessions
setenv FREESURFER_HOME $CSURF_DIR

### always use LOCAL_DIR tcl/tk/tix
setenv LOCAL_DIR     $CSURF_DIR/local
# TODO: change even old csurf to tcl8.3
setenv TCL_LIBRARY   $LOCAL_DIR/lib/tcl7.4
setenv TK_LIBRARY    $LOCAL_DIR/lib/tk4.0
setenv TIX_LIBRARY   $LOCAL_DIR/lib/tix4.1
# next 2 need for fixed truncphase tksurfer (surfer8.3 so need tixwish, too)
# kamares0.8/tcl/lib + below works w/linux-tkregister,tksurfer, but not tkmedit
#setenv TCL_LIBRARY   $LOCAL_DIR/lib/tcl8.3
#setenv TK_LIBRARY    $LOCAL_DIR/lib/tk8.3
#setenv TIX_LIBRARY   $LOCAL_DIR/lib/tix4.1.0.007

### MNI
setenv PERL5LIB         $CSURF_DIR/local/lib/mni_autoreg/perl
setenv MNI_CONFIG_PATH  $CSURF_DIR/local/lib/mni_autoreg/cfg
setenv LANG  en_US    # RH9: unsetenv'd def=en_US.UTF-8 breaks perl shellwords

### add FreeSurfer paths (re-run OK)
set arch = `uname -s`
foreach csurfpath ( \
    $CSURF_DIR/bin/noarch \
    $CSURF_DIR/bin/$arch  \
    $LOCAL_DIR/bin/$arch  \
  )
  foreach existing ($path)
    if ($csurfpath == $existing) goto skippath
  end
  set path = ($csurfpath $path)
  skippath:
end
unset csurfpath existing

### GSL (Gnu Scientific Library)
set addpath = $CSURF_DIR/gsl/lib/$arch
if (! $?LD_LIBRARY_PATH) then
  setenv LD_LIBRARY_PATH $addpath
else  # no bloat
  if ("$LD_LIBRARY_PATH" !~ *$addpath*) then
    setenv LD_LIBRARY_PATH ${addpath}:$LD_LIBRARY_PATH
  endif
endif
unset addpath

### AFNI (bump to top)
#set addpath = $CSURF_DIR/afni/lib/$arch
set addpath = /usr/local/afni/linux_xorg7
set cleanpath = ""
foreach existing ($path)
  if ($addpath != $existing) set cleanpath = ($cleanpath $existing)
end
set path = ($addpath $cleanpath)
setenv AFNI_PLUGINPATH $addpath
unset addpath cleanpath

### check for tixwish
set tixwish = $LOCAL_DIR/bin/$arch/tixwish4.1.7.4
#set tixwish = $LOCAL_DIR/bin/$arch/tixwish4.1.8.0
if (! -e $tixwish) then
  echo "FreeSurferDefs.csh: ### tixwish not found  ...install problems??"
  echo " [fullpath: $tixwish]"
endif
unset tixwish

### useful aliases for command line navigation
alias surfdoc1 "acroread $CSURF_DIR/docs/FreeSurferManual.pdf"
alias surfdoc "xpdf $CSURF_DIR/docs/FreeSurferManual.pdf"
alias sdir 'setenv SUBJECTS_DIR \!*'               # sdir newsubjects
alias fdir 'setenv FUNCTIONALS_DIR \!*'            # fdir newfuncts
alias lss 'cd $SUBJECTS_DIR;    ls -1 | grep \!*'  # lss mart (pattern match)
alias lsf 'cd $FUNCTIONALS_DIR; ls -1 | grep \!*'  # lsf MS (pattern match)
alias cds 'cd $SUBJECTS_DIR/\!*/scripts; ls'       # cds marty (into scriptdir)
alias cdf 'cd $FUNCTIONALS_DIR/\!*/*/scripts; ls'  # cdf 011325MS (ditto)

### backward
setenv MRI_DIR   $CSURF_DIR
setenv DIAG      0x4040
setenv OLDMEDCMDLINE
setenv OLDSURFCMDLINE

echo "this is FreeSurfer0.8"
