#! /usr/bin/tclsh
##############################################################################
# tksurfer script: phasecancel [calc,write phasecanceled fourier data]
##############################################################################

#### file defaults: can reset in csh script with setenv
set regphasedir    eccen
set revphasedir    eccen_r
set phasecanceldir eccencombo

#### parm defaults: can reset in csh script with setenv
set floatstem sig               ;# float file stem (eccen/polarstem overrides)
set realname 2                  ;# analyse infix
set complexname 3               ;# analyse infix
set smoothsteps 50

#### read non-cap setenv vars (or ext w/correct rgbname) to override defaults
source $env(MRI_DIR)/lib/tcl/readenv.tcl

#### stems
if ![info exists phasestem]       { set phasestem $floatstem }
if ![info exists revphasestem]    { set revphasestem $floatstem }
if ![info exists phasecancelstem] { set phasecancelstem $phasestem }

#### FOURIER 1 ####
#### read,smooth imaginary four1
puts "tksurfer: [file tail $script]: read, smooth complex Fourier comp: four1"
setfile val */$regphasedir/${phasestem}${complexname}-$hemi.w
read_binary_values
smooth_val $smoothsteps
shift_values            ;# push imaginary

#### read,smooth real four1
puts "tksurfer: [file tail $script]: read, smooth real Fourier comp: four1"
setfile val */$regphasedir/${phasestem}${realname}-$hemi.w
read_binary_values
smooth_val $smoothsteps
swap_values             ;# swap two complex numbers

#### FOURIER 2 ####
#### read,smooth imaginary four2 (reversed)
puts "tksurfer: [file tail $script]: read, smooth complex Fourier comp: four2"
setfile val */$revphasedir/${revphasestem}${complexname}-$hemi.w
read_binary_values
smooth_val $smoothsteps
shift_values            ;# push imaginary

#### read,smooth real four2 (reversed)
puts "tksurfer: [file tail $script]: read, smooth real Fourier comp: four2"
setfile val */$revphasedir/${revphasestem}${realname}-$hemi.w
read_binary_values
smooth_val $smoothsteps
swap_values             ;# swap complex number back

#### calc,write phasecancel images
puts "tksurfer: [file tail $script]: x-y to polar"
compute_angles          ;# real/imaginary => ampl/phase
puts "tksurfer: [file tail $script]: compute phasecancel"
compute_phasecancel     ;# circsubtract/geom mean scale
puts "tksurfer: [file tail $script]: write phasecancel data"
setfile val */$phasecanceldir/${phasecancelstem}${real}-$hemi.w
write_binary_values
swap_value              ;# swap real/imaginary
setfile val */$phasecanceldir/${phasecancelstem}${complexname}-$hemi.w
write_binary_values
swap_value              ;# swap back in case display

if ![info exists noexit] { exit }

