#! /usr/bin/tclsh
#############################################################################
# tksurfer script: phasemovie.tcl  [save phase movie; interactive or batch]
##############################################################################
#### uses current surface pose

#### defaults
if ![info exists rgbname] { set rgbname current }
set phasemin 0.0
set phasemax 1.0
set phasewidth 0.03
set mpgconvflag 0
if {$rgbname == "eccen"} {
  set phasemin -0.3
  set phasemax 0.5
  set phasewidth 0.05
}
if {$rgbname == "polar"} {
  set phasemin -0.5
  set phasemax 0.2
  set phasewidth 0.03
}
if {$rgbname == "downsweep"} {
  set phasemin 0.2
  set phasemax 0.52
  set phasewidth 0.03
}
if {$rgbname == "upsweep"} {
  set phasemin 0.3
  set phasemax 0.7
  set phasewidth 0.03
}
set phasesteps 50
set phasecontour_bright 255

#### two ways to override defaults
if [winfo viewable .] {    ;# make popup
  tmpcontrols "PHASE MOVIE" { rgbname phasemin phasemax phasewidth phasesteps }
  if {!$userok} { return }
} else {                   ;# batch scripts; re-read env to override defaults
  source $env(MRI_DIR)/lib/tcl/readenv.tcl
}

if [info exists nosave] {
  puts "phasemovie.tcl: ### nosave is set (csh:unsetenv or tcl:unset to save)"
}
puts "phasemovie.tcl: making phasemovie"
puts "     files: $rgbname-phmov-$hemi-{currsurface}-??,ph=?.??.rgb"
prompt

#### save phasemovie (phase in {-0.5,0.5}; before rev, inv)
set flatext flat
set incr [expr ($phasemax-$phasemin)/$phasesteps]
set phasecontourmodflag TRUE
set phasecontourflag TRUE
set i 0
if {$incr > 0.0} { 
  set phasetest {$phase < $phasemax}
} else {
  set phasetest {$phase > $phasemax}
}
for {set phase $phasemin} $phasetest {set phase [expr $phase+$incr]} {
  # surfer divides phase across max/min boundary--just wrap into [-0.5,0.5]
  set phasecontour_min [expr $phase - $phasewidth/2.0]
  if { $phasecontour_min > 0.5 } {
    set phasecontour_min [expr $phasecontour_min - 1.0]
  }
  if { $phasecontour_min < -0.5 } {
    set phasecontour_min [expr $phasecontour_min + 1.0]
  }
  set phasecontour_max [expr $phase + $phasewidth/2.0]
  if { $phasecontour_max > 0.5 } {
    set phasecontour_max [expr $phasecontour_max - 1.0]
  }
  if { $phasecontour_max < -0.5 } {
    set phasecontour_max [expr $phasecontour_max + 1.0]
  }
  redraw
  set rgbnum [format "%02d" $i]
  set phanum ph=[format "%+2.2f" $phase]
  # name doesn't incl lat/med/etc, real patchname => OK for tmp/overwrite
  if {$flag2d} {
    if { [string match *nomid* $patchname] } { set flatext nomid }
    save_rgb_named $rgbname-phmov-$hemi-$flatext-$rgbnum,$phanum.rgb
    set currext $flatext
  } else {
    save_rgb_named $rgbname-phmov-$hemi-$ext-$rgbnum,$phanum.rgb
    set currext $ext
  }
  incr i
}
set phasecontourflag FALSE

# convert to mpg, delete rgbs (50X compression)
if {$mpgconvflag} {
  set rgblist \
    [lsort [glob $session/rgb/$rgbname-phmov-$hemi-$currext-??,ph=??.??.rgb]]
  puts "phasemovie.tcl: convert to mpg w/rgb2mpg, delete rgb's"
  puts "phasemovie.tcl: rgb2mpg 1.0 1 1 <currrname>*.rgb ~/mpg/<samename>.mpg"
  eval exec rgb2mpg 1.0 1 1 $rgblist $session/mpg/$rgbname-phmov-$hemi-$ext.mpg
  eval exec rm $rgblist
}
